/*
 * Copyright (c) 2012, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <avr/pgmspace.h>

#include "usart.h"
#include "response.h"

// All strings are placed in program memory
static const char resp_0[] PROGMEM = "voltsrc 1.0 ready";
static const char resp_1[] PROGMEM = "Channel set";
static const char resp_2[] PROGMEM = "Invalid command";
static const char resp_3[] PROGMEM = "Invalid parameter";
static const char resp_4[] PROGMEM = "Missing parameter";
static const char resp_5[] PROGMEM = "+OK ";
static const char resp_6[] PROGMEM = "-ERR ";
static const char resp_7[] PROGMEM = "\r\n";

// String table is also placed in program memory
static PGM_P const resp_table[] PROGMEM =
{
	resp_0,
	resp_1,
	resp_2,
	resp_3,
	resp_4,
	resp_5,
	resp_6,
	resp_7,
};

static void resp_print(enum resp_id id)
{
	PGM_P p;
	char c;

	p = (PGM_P)pgm_read_word(&(resp_table[id]));
	while ((c = pgm_read_byte(p)) != 0) {
		usart_tx_byte(c);
		p++;
	}
}

void resp_pos(enum resp_id id)
{
	resp_print(RESP_POSITIVE);
	resp_print(id);
	resp_print(RESP_CRLF);
}

void resp_neg(enum resp_id id)
{
	resp_print(RESP_NEGATIVE);
	resp_print(id);
	resp_print(RESP_CRLF);
}

